@@ -127,6 +127,17 @@ static int check_conn_header(uint32_t header, const struct main_config *config,
 					"Waypipe client is rejecting connection, Waypipe client was not configured for the VP9 video coding format requested by the Waypipe server");
 			return -1;
 		}
+	} else if ((header & CONN_VIDEO_MASK) == CONN_AV1_VIDEO) {
+		if (!config->video_if_possible) {
+			snprintf(err, err_size,
+					"Waypipe client is rejecting connection, Waypipe client was not run with video encoding enabled, unlike Waypipe server");
+			return -1;
+		}
+		if (config->video_fmt != VIDEO_AV1) {
+			snprintf(err, err_size,
+					"Waypipe client is rejecting connection, Waypipe client was not configured for the AV1 video coding format requested by the Waypipe server");
+			return -1;
+		}
 	} else if ((header & CONN_VIDEO_MASK) == CONN_NO_VIDEO) {
 		if (config->video_if_possible) {
 			snprintf(err, err_size,
@@ -760,8 +771,10 @@ int run_client(int cwd_fd, const char *sock_folder_name, int sock_folder_fd,
 		retcode = run_multi_client(cwd_fd, channelsock, &eol_pid,
 				config, display_path);
 	}
-	unlink_at_folder(cwd_fd, sock_folder_fd, sock_folder_name,
-			sock_filename);
+	if (!config->vsock) {
+		unlink_at_folder(cwd_fd, sock_folder_fd, sock_folder_name,
+				sock_filename);
+	}
 	int cleanup_type = shutdown_flag ? WNOHANG : 0;
 
 	int status = -1;
@@ -776,7 +789,9 @@ fail:
 	if (eol_pid) {
 		waitpid(eol_pid, NULL, 0);
 	}
-	unlink_at_folder(cwd_fd, sock_folder_fd, sock_folder_name,
-			sock_filename);
+	if (!config->vsock) {
+		unlink_at_folder(cwd_fd, sock_folder_fd, sock_folder_name,
+				sock_filename);
+	}
 	return EXIT_FAILURE;
 }
