@@ -36,6 +36,10 @@
 
 #include <protocols.h>
 
+#ifndef DRM_FORMAT_MOD_INVALID
+#define DRM_FORMAT_MOD_INVALID 0x00ffffffffffffffULL
+#endif
+
 struct obj_wl_shm_pool {
 	struct wp_object base;
 	struct shadow_fd *owned_buffer;
@@ -232,6 +236,23 @@ static const struct wp_interface *const non_global_interfaces[] = {
 		&intf_zwp_primary_selection_source_v1,
 };
 
+static void cleanup_dmabuf_params_fds(struct obj_zwp_linux_dmabuf_params *r)
+{
+	// Sometimes multiple entries point to the same buffer
+	for (int i = 0; i < MAX_DMABUF_PLANES; i++) {
+		int fd = r->add[i].fd;
+		if (fd != -1) {
+			checked_close(fd);
+
+			for (int k = 0; k < MAX_DMABUF_PLANES; k++) {
+				if (fd == r->add[k].fd) {
+					r->add[k].fd = -1;
+				}
+			}
+		}
+	}
+}
+
 void destroy_wp_object(struct wp_object *object)
 {
 	if (object->type == &intf_wl_shm_pool) {
@@ -267,17 +288,8 @@ void destroy_wp_object(struct wp_object *object)
 			if (r->add[i].buffer) {
 				shadow_decref_protocol(r->add[i].buffer);
 			}
-			// Sometimes multiple entries point to the same buffer
-			if (r->add[i].fd != -1) {
-				checked_close(r->add[i].fd);
-
-				for (int k = 0; k < MAX_DMABUF_PLANES; k++) {
-					if (r->add[i].fd == r->add[k].fd) {
-						r->add[k].fd = -1;
-					}
-				}
-			}
 		}
+		cleanup_dmabuf_params_fds(r);
 	} else if (object->type == &intf_zwlr_export_dmabuf_frame_v1) {
 		struct obj_wlr_export_dmabuf_frame *r =
 				(struct obj_wlr_export_dmabuf_frame *)object;
@@ -425,6 +437,7 @@ void do_wl_registry_evt_global(struct context *ctx, uint32_t name,
 	// requires novel fd translation, not yet supported
 	unsupported |= !strcmp(
 			interface, "zwp_linux_explicit_synchronization_v1");
+	unsupported |= !strcmp(interface, "wp_linux_drm_syncobj_manager_v1");
 	if (unsupported) {
 		wp_debug("Hiding %s advertisement, unsupported", interface);
 		ctx->drop_this_msg = true;
@@ -955,8 +968,8 @@ void do_wl_keyboard_evt_keymap(
 		return;
 	}
 
-	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render, fd,
-			FDC_FILE, fdsz, NULL, false);
+	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render,
+			&ctx->g->threads, fd, FDC_FILE, fdsz, NULL, false);
 	if (!sfd) {
 		wp_error("Failed to create shadow for keymap fd=%d", fd);
 		return;
@@ -996,8 +1009,8 @@ void do_wl_shm_req_create_pool(
 		return;
 	}
 
-	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render, fd,
-			FDC_FILE, fdsz, NULL, false);
+	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render,
+			&ctx->g->threads, fd, FDC_FILE, fdsz, NULL, false);
 	if (!sfd) {
 		return;
 	}
@@ -1248,7 +1261,7 @@ void do_wl_drm_req_create_prime_buffer(struct context *ctx,
 	};
 
 	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render,
-			name, FDC_DMABUF, 0, &info, false);
+			&ctx->g->threads, name, FDC_DMABUF, 0, &info, false);
 	if (!sfd) {
 		return;
 	}
@@ -1339,12 +1352,17 @@ void do_zwp_linux_buffer_params_v1_evt_created(
 					i);
 			continue;
 		}
-		buf->dmabuf_buffers[i] =
-				shadow_incref_protocol(params->add[i].buffer);
+		// Move protocol reference from `params` to `buf`
+		// (The params object can only be used to create one buffer,
+		// so this ensures that if the params object leaks, the
+		// shadow_fd does not leak as well.)
+		buf->dmabuf_buffers[i] = params->add[i].buffer;
 		buf->dmabuf_offsets[i] = params->add[i].offset;
 		buf->dmabuf_strides[i] = params->add[i].stride;
 		buf->dmabuf_modifiers[i] = params->add[i].modifier;
+		params->add[i].buffer = NULL;
 	}
+	cleanup_dmabuf_params_fds(params);
 	buf->dmabuf_flags = params->create_flags;
 	buf->dmabuf_width = params->create_width;
 	buf->dmabuf_height = params->create_height;
@@ -1453,8 +1471,8 @@ void do_zwp_linux_buffer_params_v1_req_create(struct context *ctx,
 		/* note: the``info` provided includes the incoming/as-if stride
 		 * data. */
 		struct shadow_fd *sfd = translate_fd(&ctx->g->map,
-				&ctx->g->render, params->add[i].fd, res_type, 0,
-				&info, false);
+				&ctx->g->render, &ctx->g->threads,
+				params->add[i].fd, res_type, 0, &info, false);
 		if (!sfd) {
 			continue;
 		}
@@ -1664,8 +1682,8 @@ void do_zwp_linux_dmabuf_feedback_v1_evt_format_table(
 				fd, fdcat_to_str(fdtype), fdsz, size);
 		return;
 	}
-	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render, fd,
-			FDC_FILE, size, NULL, false);
+	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render,
+			&ctx->g->threads, fd, FDC_FILE, size, NULL, false);
 	if (!sfd) {
 		return;
 	}
@@ -1853,8 +1871,8 @@ void do_zwlr_export_dmabuf_frame_v1_evt_object(struct context *ctx,
 			.modifier = frame->modifier};
 	info.using_planes[index] = true;
 
-	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render, fd,
-			FDC_DMABUF, 0, &info, false);
+	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render,
+			&ctx->g->threads, fd, FDC_DMABUF, 0, &info, false);
 	if (!sfd) {
 		return;
 	}
@@ -1891,14 +1909,14 @@ void do_zwlr_export_dmabuf_frame_v1_evt_ready(struct context *ctx,
 		}
 	}
 }
-static void translate_data_transfer_fd(struct context *context, int32_t fd)
+static void translate_data_transfer_fd(struct context *ctx, int32_t fd)
 {
 	/* treat the fd as a one-way pipe, even if it is e.g. a file or
 	 * socketpair, with additional properties. The fd being sent
 	 * around should be, according to the protocol, only written into and
 	 * closed */
-	(void)translate_fd(&context->g->map, &context->g->render, fd, FDC_PIPE,
-			0, NULL, true);
+	(void)translate_fd(&ctx->g->map, &ctx->g->render, &ctx->g->threads, fd,
+			FDC_PIPE, 0, NULL, true);
 }
 void do_gtk_primary_selection_offer_req_receive(
 		struct context *ctx, const char *mime_type, int fd)
@@ -1964,8 +1982,8 @@ void do_zwlr_gamma_control_v1_req_set_gamma(struct context *ctx, int fd)
 				fdcat_to_str(fdtype));
 		return;
 	}
-	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render, fd,
-			FDC_FILE, fdsz, NULL, false);
+	struct shadow_fd *sfd = translate_fd(&ctx->g->map, &ctx->g->render,
+			&ctx->g->threads, fd, FDC_FILE, fdsz, NULL, false);
 	if (!sfd) {
 		return;
 	}
@@ -1975,4 +1993,31 @@ void do_zwlr_gamma_control_v1_req_set_gamma(struct context *ctx, int fd)
 	sfd->has_owner = true;
 }
 
+#define MSGNO_XDG_TOPLEVEL_REQ_SET_TITLE 2
+void do_xdg_toplevel_req_set_title(struct context *ctx, const char *str)
+{
+	if (!ctx->g->config->title_prefix) {
+		return;
+	}
+	size_t prefix_len = strlen(ctx->g->config->title_prefix);
+	if (4 + (int)prefix_len >= ctx->message_available_space) {
+		wp_error("Not enough space (%d left, at most %d needed) to prepend title prefix",
+				ctx->message_available_space, 4 + prefix_len);
+		return;
+	}
+	size_t title_len = strlen(str);
+	size_t str_part = alignz(prefix_len + title_len + 1, 4);
+
+	ctx->message[1] = message_header_2((uint32_t)str_part + 12,
+			MSGNO_XDG_TOPLEVEL_REQ_SET_TITLE);
+	ctx->message[2] = (uint32_t)(prefix_len + title_len + 1);
+	char *v = (char *)&ctx->message[3];
+	// Using memmove, as str=&ctx->message[3]
+	memmove(v + prefix_len, v, title_len);
+	memset(v + prefix_len + title_len, 0,
+			str_part - prefix_len - title_len);
+	memcpy(v, ctx->g->config->title_prefix, prefix_len);
+	ctx->message_length = 12 + (int)str_part;
+}
+
 const struct wp_interface *the_display_interface = &intf_wl_display;
