@@ -41,6 +41,11 @@ struct main_config {
 	enum video_coding_fmt video_fmt;
 	bool prefer_hwvideo;
 	bool old_video_mode;
+	bool vsock;
+	uint32_t vsock_cid;
+	uint32_t vsock_port;
+	bool vsock_to_host;
+	const char *title_prefix;
 };
 struct globals {
 	const struct main_config *config;
