@@ -533,9 +533,9 @@ static void uncompress_buffer(struct thread_pool *pool, struct comp_ctx *ctx,
 }
 
 struct shadow_fd *translate_fd(struct fd_translation_map *map,
-		struct render_data *render, int fd, enum fdcat type,
-		size_t file_sz, const struct dmabuf_slice_data *info,
-		bool force_pipe_iw)
+		struct render_data *render, struct thread_pool *threads, int fd,
+		enum fdcat type, size_t file_sz,
+		const struct dmabuf_slice_data *info, bool force_pipe_iw)
 {
 	struct shadow_fd *sfd = get_shadow_for_local_fd(map, fd);
 	if (sfd) {
@@ -657,7 +657,7 @@ struct shadow_fd *translate_fd(struct fd_translation_map *map,
 		if (!sfd->dmabuf_bo) {
 			return sfd;
 		}
-		if (setup_video_encode(sfd, render) == -1) {
+		if (setup_video_encode(sfd, render, threads->nthreads) == -1) {
 			wp_error("Video encoding setup failed for RID=%d",
 					sfd->remote_id);
 		}
@@ -718,6 +718,7 @@ static void worker_run_compress_diff(
 {
 	struct shadow_fd *sfd = task->sfd;
 	struct thread_pool *pool = local->pool;
+	size_t diffsize = (size_t)-1;
 
 	size_t damage_space = 0;
 	for (int i = 0; i < task->damage_len; i++) {
@@ -798,7 +799,7 @@ static void worker_run_compress_diff(
 		source = sfd->dmabuf_warped;
 	}
 
-	size_t diffsize = construct_diff_core(pool->diff_func,
+	diffsize = construct_diff_core(pool->diff_func,
 			pool->diff_alignment_bits, task->damage_intervals,
 			task->damage_len, sfd->mem_mirror, source, diff_target);
 	size_t ntrailing = 0;
@@ -1144,7 +1145,7 @@ static void add_dmabuf_create_request_v2(struct transfer_queue *transfers,
 	header->file_size = (uint32_t)sfd->buffer_size;
 	header->remote_id = sfd->remote_id;
 	header->size_and_type = transfer_header(actual_len, variant);
-	header->vid_flags = (fmt == VIDEO_H264) ? DMAVID_H264 : DMAVID_VP9;
+	header->vid_flags = (uint32_t)fmt;
 
 	memcpy(data + sizeof(*header), &sfd->dmabuf_info,
 			sizeof(struct dmabuf_slice_data));
@@ -1696,15 +1697,16 @@ int apply_update(struct fd_translation_map *map, struct thread_pool *threads,
 			memcpy(&sfd->dmabuf_info,
 					msg->data + sizeof(struct wmsg_open_dmavid),
 					sizeof(struct dmabuf_slice_data));
-			if ((header.vid_flags & 0xff) == DMAVID_H264) {
-				sfd->video_fmt = VIDEO_H264;
-			} else if ((header.vid_flags & 0xff) == DMAVID_VP9) {
-				sfd->video_fmt = VIDEO_VP9;
+			uint32_t vid_type = header.vid_flags & 0xff;
+			if (vid_type == (uint32_t)VIDEO_H264 ||
+					vid_type == (uint32_t)VIDEO_VP9 ||
+					vid_type == (uint32_t)VIDEO_AV1) {
+				sfd->video_fmt =
+						(enum video_coding_fmt)vid_type;
 			} else {
-				sfd->video_fmt = VIDEO_H264;
-				wp_error("Unidentified video format for RID=%d",
-						sfd->remote_id);
-				return 0;
+				wp_error("Unidentified video format %u for RID=%d",
+						vid_type, sfd->remote_id);
+				return ERR_FATAL;
 			}
 		}
 
@@ -1765,15 +1767,16 @@ int apply_update(struct fd_translation_map *map, struct thread_pool *threads,
 			memcpy(&sfd->dmabuf_info,
 					msg->data + sizeof(struct wmsg_open_dmavid),
 					sizeof(struct dmabuf_slice_data));
-			if ((header.vid_flags & 0xff) == DMAVID_H264) {
-				sfd->video_fmt = VIDEO_H264;
-			} else if ((header.vid_flags & 0xff) == DMAVID_VP9) {
-				sfd->video_fmt = VIDEO_VP9;
+			uint32_t vid_type = header.vid_flags & 0xff;
+			if (vid_type == (uint32_t)VIDEO_H264 ||
+					vid_type == (uint32_t)VIDEO_VP9 ||
+					vid_type == (uint32_t)VIDEO_AV1) {
+				sfd->video_fmt =
+						(enum video_coding_fmt)vid_type;
 			} else {
-				sfd->video_fmt = VIDEO_H264;
-				wp_error("Unidentified video format for RID=%d",
+				wp_error("Unidentified video format %u for RID=%d",
 						sfd->remote_id);
-				return 0;
+				return ERR_FATAL;
 			}
 		}
 
@@ -1790,7 +1793,7 @@ int apply_update(struct fd_translation_map *map, struct thread_pool *threads,
 		}
 		sfd->fd_local = export_dmabuf(sfd->dmabuf_bo);
 
-		if (setup_video_encode(sfd, render) == -1) {
+		if (setup_video_encode(sfd, render, threads->nthreads) == -1) {
 			wp_error("Video encoding setup failed for RID=%d",
 					sfd->remote_id);
 		}
