@@ -83,6 +83,7 @@ static const char usage_string[] =
 		"                         server default: /tmp/waypipe-server.sock\n"
 		"                         client default: /tmp/waypipe-client.sock\n"
 		"                         ssh: sets the prefix for the socket path\n"
+		"                         vsock: [[s]CID:]port\n"
 		"      --version        print waypipe version and exit\n"
 		"      --allow-tiled    allow gpu buffers (DMABUFs) with format modifiers\n"
 		"      --control C      server,ssh: set control pipe to reconnect server\n"
@@ -92,9 +93,11 @@ static const char usage_string[] =
 		"      --remote-bin R   ssh: set the remote waypipe binary. default: waypipe\n"
 		"      --login-shell    server: if server CMD is empty, run a login shell\n"
 		"      --threads T      set thread pool size, default=hardware threads/2\n"
+		"      --title-prefix P prepend P to all window titles\n"
 		"      --unlink-socket  server: unlink the socket that waypipe connects to\n"
 		"      --video[=V]      compress certain linear dmabufs only with a video codec\n"
-		"                         V is list of options: sw,hw,bpf=1.2e5,h264,vp9\n"
+		"                         V is list of options: sw,hw,bpf=1.2e5,h264,vp9,av1\n"
+		"      --vsock          use vsock instead of unix socket\n"
 		"\n";
 
 static int usage(int retcode)
@@ -324,6 +327,8 @@ static int parse_video_string(const char *str, struct main_config *config)
 			config->video_fmt = VIDEO_H264;
 		} else if (!strcmp(part, "vp9")) {
 			config->video_fmt = VIDEO_VP9;
+		} else if (!strcmp(part, "av1")) {
+			config->video_fmt = VIDEO_AV1;
 		} else if (!strcmp(part, "hw")) {
 			config->prefer_hwvideo = true;
 		} else if (!strcmp(part, "sw")) {
@@ -345,6 +350,56 @@ static int parse_video_string(const char *str, struct main_config *config)
 }
 #endif
 
+#ifdef HAS_VSOCK
+static int parse_vsock_addr(const char *str, struct main_config *config)
+{
+	char tmp[128];
+	size_t l = strlen(str);
+	if (l >= 127) {
+		return -1;
+	}
+	memcpy(tmp, str, l + 1);
+
+	char *port = strchr(tmp, ':');
+	if (port) {
+		char *cid = tmp;
+		port[0] = 0;
+		port = port + 1;
+
+		size_t cid_len = strlen(cid);
+		if (cid_len > 0) {
+			if (cid[0] == 's') {
+				if (cid_len < 2) {
+					return -1;
+				}
+				config->vsock_to_host = true;
+				if (parse_uint32(cid + 1, &config->vsock_cid) ==
+						-1) {
+					return -1;
+				}
+			} else {
+				config->vsock_to_host = false;
+				if (parse_uint32(cid, &config->vsock_cid) ==
+						-1) {
+					return -1;
+				}
+			}
+		}
+	} else {
+		port = tmp;
+	}
+
+	if (parse_uint32(port, &config->vsock_port) == -1) {
+		return -1;
+	}
+
+	if (config->vsock_port <= 0) {
+		return -1;
+	}
+	return 0;
+}
+#endif
+
 static const char *feature_names[] = {
 		"lz4",
 		"zstd",
@@ -393,6 +448,8 @@ static const bool feature_flags[] = {
 #define ARG_CONTROL 1010
 #define ARG_WAYPIPE_BINARY 1011
 #define ARG_BENCH_TEST_SIZE 1012
+#define ARG_VSOCK 1013
+#define ARG_TITLE_PREFIX 1014
 
 static const struct option options[] = {
 		{"compress", required_argument, NULL, 'c'},
@@ -414,6 +471,8 @@ static const struct option options[] = {
 		{"display", required_argument, NULL, ARG_DISPLAY},
 		{"control", required_argument, NULL, ARG_CONTROL},
 		{"test-size", required_argument, NULL, ARG_BENCH_TEST_SIZE},
+		{"vsock", no_argument, NULL, ARG_VSOCK},
+		{"title-prefix", required_argument, NULL, ARG_TITLE_PREFIX},
 		{0, 0, NULL, 0}};
 struct arg_permissions {
 	int val;
@@ -421,18 +480,15 @@ struct arg_permissions {
 };
 #define ALL_MODES (uint32_t) - 1
 static const struct arg_permissions arg_permissions[] = {
-		{'c', MODE_SSH | MODE_CLIENT | MODE_SERVER},
-		{'d', ALL_MODES},
-		{'h', MODE_FAIL},
-		{'n', MODE_SSH | MODE_CLIENT | MODE_SERVER},
+		{'c', MODE_SSH | MODE_CLIENT | MODE_SERVER}, {'d', ALL_MODES},
+		{'h', MODE_FAIL}, {'n', MODE_SSH | MODE_CLIENT | MODE_SERVER},
 		{'o', MODE_SSH | MODE_CLIENT | MODE_SERVER},
 		{'s', MODE_SSH | MODE_CLIENT | MODE_SERVER},
 		{ARG_VERSION, MODE_FAIL},
 		{ARG_ALLOW_TILED, MODE_SSH | MODE_CLIENT | MODE_SERVER},
 		{ARG_UNLINK, MODE_SERVER},
 		{ARG_DRMNODE, MODE_SSH | MODE_CLIENT | MODE_SERVER},
-		{ARG_REMOTENODE, MODE_SSH},
-		{ARG_WAYPIPE_BINARY, MODE_SSH},
+		{ARG_REMOTENODE, MODE_SSH}, {ARG_WAYPIPE_BINARY, MODE_SSH},
 		{ARG_LOGIN_SHELL, MODE_SERVER},
 		{ARG_VIDEO, MODE_SSH | MODE_CLIENT | MODE_SERVER},
 		{ARG_HWVIDEO, MODE_SSH | MODE_CLIENT | MODE_SERVER},
@@ -441,7 +497,8 @@ static const struct arg_permissions arg_permissions[] = {
 		{ARG_DISPLAY, MODE_SSH | MODE_SERVER},
 		{ARG_CONTROL, MODE_SSH | MODE_SERVER},
 		{ARG_BENCH_TEST_SIZE, MODE_BENCH},
-};
+		{ARG_VSOCK, MODE_SSH | MODE_CLIENT | MODE_SERVER},
+		{ARG_TITLE_PREFIX, MODE_SSH | MODE_CLIENT | MODE_SERVER}};
 
 /* envp is nonstandard, so use environ */
 extern char **environ;
@@ -464,16 +521,27 @@ int main(int argc, char **argv)
 	char *socketpath = NULL;
 	uint32_t bench_test_size = (1u << 22) + 13;
 
-	struct main_config config = {.n_worker_threads = 0,
+	struct main_config config = {
+			.n_worker_threads = 0,
 			.drm_node = NULL,
+#ifdef HAS_LZ4
+			.compression = COMP_LZ4,
+#else
 			.compression = COMP_NONE,
+#endif
 			.compression_level = 0,
 			.no_gpu = false,
 			.only_linear_dmabuf = true,
 			.video_if_possible = false,
 			.video_bpf = 0,
 			.video_fmt = VIDEO_H264,
-			.prefer_hwvideo = false};
+			.prefer_hwvideo = false,
+			.vsock = false,
+			.vsock_cid = 2,         /* VMADDR_CID_HOST */
+			.vsock_to_host = false, /* VMADDR_FLAG_TO_HOST */
+			.vsock_port = 0,
+			.title_prefix = NULL,
+	};
 
 	/* We do not parse any getopt arguments happening after the mode choice
 	 * string, so as not to interfere with them. */
@@ -637,6 +705,25 @@ int main(int argc, char **argv)
 				fail = true;
 			}
 		} break;
+		case ARG_VSOCK:
+#ifdef HAS_VSOCK
+			config.vsock = true;
+			break;
+#else
+			fprintf(stderr, "Option --vsock not allowed: this copy of Waypipe was not built with support for Linux VM sockets.\n");
+			return EXIT_FAILURE;
+#endif
+		case ARG_TITLE_PREFIX:
+			if (!is_utf8(optarg)) {
+				fprintf(stderr, "Title prefix argument must be valid UTF-8.\n");
+				return EXIT_FAILURE;
+			}
+			if (strlen(optarg) > 128) {
+				fprintf(stderr, "Title prefix is too long (>128 bytes).\n");
+				return EXIT_FAILURE;
+			}
+			config.title_prefix = optarg;
+			break;
 		default:
 			fail = true;
 			break;
@@ -699,6 +786,20 @@ int main(int argc, char **argv)
 		config.video_bpf = config.prefer_hwvideo ? 360000 : 120000;
 	}
 
+#ifdef HAS_VSOCK
+	if (config.vsock) {
+		if (socketpath == NULL) {
+			fprintf(stderr, "Socket option (-s, --socket) is required when vsock is enabled\n");
+			return EXIT_FAILURE;
+		}
+		if (parse_vsock_addr(socketpath, &config) == -1) {
+			fprintf(stderr, "Invalid vsock address specification: '%s' does not match form [[s]CID:]port\n",
+					socketpath);
+			return EXIT_FAILURE;
+		}
+	}
+#endif
+
 	if (debug) {
 		log_funcs[0] = log_handler;
 	}
@@ -767,17 +868,31 @@ int main(int argc, char **argv)
 
 			int nmaxclients = oneshot ? 1 : 128;
 			int client_folder_fd = -1, channelsock = -1;
-			if (setup_nb_socket(cwd_fd, client_sock_path,
-					    nmaxclients, &client_folder_fd,
-					    &channelsock) == -1) {
-				return EXIT_FAILURE;
+
+			if (!config.vsock) {
+				if (setup_nb_socket(cwd_fd, client_sock_path,
+						    nmaxclients,
+						    &client_folder_fd,
+						    &channelsock) == -1) {
+					return EXIT_FAILURE;
+				}
+			} else {
+#ifdef HAS_VSOCK
+				if (listen_on_vsock(config.vsock_port,
+						    nmaxclients,
+						    &channelsock) == -1) {
+					return EXIT_FAILURE;
+				}
+#endif
 			}
 			ret = run_client(cwd_fd, client_sock_path.folder,
 					client_folder_fd,
 					client_sock_path.filename->sun_path,
 					&config, oneshot, wayland_socket, 0,
 					channelsock);
-			checked_close(client_folder_fd);
+			if (!config.vsock) {
+				checked_close(client_folder_fd);
+			}
 		}
 	} else if (mode == MODE_SERVER) {
 		char *const *app_argv = (char *const *)argv;
@@ -868,18 +983,34 @@ int main(int argc, char **argv)
 
 		int nmaxclients = oneshot ? 1 : 128;
 		int channel_folder_fd = -1, channelsock = -1;
-		if (setup_nb_socket(cwd_fd, client_sock_path, nmaxclients,
-				    &channel_folder_fd, &channelsock) == -1) {
-			close(cwd_fd);
-			return EXIT_FAILURE;
-		}
-		if (set_cloexec(channelsock) == -1 ||
-				set_cloexec(channel_folder_fd) == -1) {
-			wp_error("Failed to make client socket or its folder cloexec");
-			close(channel_folder_fd);
-			close(channelsock);
-			close(cwd_fd);
-			return EXIT_FAILURE;
+		if (!config.vsock) {
+			if (setup_nb_socket(cwd_fd, client_sock_path,
+					    nmaxclients, &channel_folder_fd,
+					    &channelsock) == -1) {
+				close(cwd_fd);
+				return EXIT_FAILURE;
+			}
+			if (set_cloexec(channelsock) == -1 ||
+					set_cloexec(channel_folder_fd) == -1) {
+				wp_error("Failed to make client socket or its folder cloexec");
+				close(channel_folder_fd);
+				close(channelsock);
+				close(cwd_fd);
+				return EXIT_FAILURE;
+			}
+		} else {
+#ifdef HAS_VSOCK
+			if (listen_on_vsock(config.vsock_port, nmaxclients,
+					    &channelsock) == -1) {
+				return EXIT_FAILURE;
+			}
+			if (set_cloexec(channelsock) == -1) {
+				wp_error("Failed to make client socket or its folder cloexec");
+				close(channelsock);
+				close(cwd_fd);
+				return EXIT_FAILURE;
+			}
+#endif
 		}
 
 		pid_t conn_pid;
@@ -888,10 +1019,16 @@ int main(int argc, char **argv)
 			char serversock[256];
 			char video_str[140];
 			char remote_display[20];
-			sprintf(serversock, "%s-server-%s.sock", socketpath,
-					rbytes);
-			sprintf(linkage, "%s-server-%s.sock:%s-client-%s.sock",
-					socketpath, rbytes, socketpath, rbytes);
+			if (!config.vsock) {
+				sprintf(serversock, "%s-server-%s.sock",
+						socketpath, rbytes);
+				sprintf(linkage,
+						"%s-server-%s.sock:%s-client-%s.sock",
+						socketpath, rbytes, socketpath,
+						rbytes);
+			} else {
+				sprintf(serversock, "%d", config.vsock_port);
+			}
 			sprintf(remote_display, "wayland-%s", rbytes);
 			if (!wayland_display) {
 				wayland_display = remote_display;
@@ -919,8 +1056,10 @@ int main(int argc, char **argv)
 				 * `log_handler`. */
 				arglist[offset++] = "-t";
 			}
-			arglist[offset++] = "-R";
-			arglist[offset++] = linkage;
+			if (!config.vsock) {
+				arglist[offset++] = "-R";
+				arglist[offset++] = linkage;
+			}
 			for (int i = 0; i <= dstidx; i++) {
 				arglist[offset + i] = argv[i];
 			}
@@ -933,7 +1072,7 @@ int main(int argc, char **argv)
 			}
 
 			/* Always send the compression flag, because the default
-			 * will be changed from NONE to LZ4. */
+			 * was be changed from NONE to LZ4. */
 			arglist[dstidx + 1 + offset++] = "-c";
 			if (!comp_string) {
 				switch (config.compression) {
@@ -956,11 +1095,22 @@ int main(int argc, char **argv)
 			}
 			if (config.video_if_possible) {
 				if (!config.old_video_mode) {
+					char *vid_type = NULL;
+					switch (config.video_fmt) {
+					case VIDEO_H264:
+						vid_type = "h264";
+						break;
+					case VIDEO_VP9:
+						vid_type = "vp9";
+						break;
+					case VIDEO_AV1:
+						vid_type = "av1";
+						break;
+					}
+
 					sprintf(video_str,
 							"--video=%s,%s,bpf=%d",
-							config.video_fmt == VIDEO_H264
-									? "h264"
-									: "vp9",
+							vid_type,
 							config.prefer_hwvideo
 									? "hw"
 									: "sw",
@@ -996,12 +1146,14 @@ int main(int argc, char **argv)
 			arglist[dstidx + 1 + offset++] = serversock;
 			arglist[dstidx + 1 + offset++] = "--display";
 			arglist[dstidx + 1 + offset++] = wayland_display;
+			if (config.vsock) {
+				arglist[dstidx + 1 + offset++] = "--vsock";
+			}
 			arglist[dstidx + 1 + offset++] = "server";
 			for (int i = dstidx + 1; i < argc; i++) {
 				arglist[offset + i] = argv[i];
 			}
 			arglist[argc + offset] = NULL;
-
 			int err = posix_spawnp(&conn_pid, arglist[0], NULL,
 					NULL, arglist, environ);
 			if (err) {
@@ -1019,7 +1171,9 @@ int main(int argc, char **argv)
 				channel_folder_fd,
 				client_sock_path.filename->sun_path, &config,
 				oneshot, wayland_socket, conn_pid, channelsock);
-		checked_close(channel_folder_fd);
+		if (!config.vsock) {
+			checked_close(channel_folder_fd);
+		}
 	}
 	checked_close(cwd_fd);
 	check_unclosed_fds();
